"""

Test for protectionlayer objects

Meant to be subclassed

"""
from django.contrib.auth.models import User
from django.test import TestCase

from installations import models as installations_models

from utils import tests as utils_tests
from utils.versioning.models import WorkVersion, Version

class ProtectionLayerTest(TestCase):
    fixtures = ['dummyusers', 'installations']
##    sc_model = Scenario
##    pl_model = ProtectionLayer
##    namespace = "degradations"

    def setUp(self):
        self.setPermissions()
        defaultuser = User.objects.get(pk=1)
        self.scope = installations_models.Scope.objects.filter(version__isnull=True)[0]
        self.scenario = self.sc_model(name='scenario', scope=self.scope)
        self.scenario.save(defaultuser)
        self.protectionlayer = self.pl_model(name='Interlock')
        self.protectionlayer.save(defaultuser, parent=None, treeelementkwargs={'scenario': self.scenario})
        self.measure = self.me_model(name='M1', type='PL')
        self.measure.save(defaultuser)
        self.plme = self.plme_model(protectionlayer=self.protectionlayer, measure=self.measure)
        self.plme.save(defaultuser)
        self.measure2 = self.me_model(name='M2', type='PL')
        self.measure2.save(defaultuser)
        self.suggestionlist = self.sl_model(name='SL')
        self.suggestionlist.save(defaultuser)
        self.slscenario = self.sc_model(name='sl scenario', suggestionlist=self.suggestionlist)
        self.slscenario.save(defaultuser)

    def test_model(self):
        defaultuser = User.objects.get(pk=1)
        protectionlayer = self.pl_model(name='PL')
        protectionlayer.save(defaultuser, parent=None, treeelementkwargs={'scenario': self.scenario})
        self.assertEquals(protectionlayer.SL, False)
        protectionlayer.delete(defaultuser)
        protectionlayer = self.pl_model(name='PL', SL=True)
        protectionlayer.save(defaultuser, parent=None, treeelementkwargs={'scenario': self.slscenario})
        self.assertEquals(protectionlayer.SL, True)
        protectionlayer.delete(defaultuser)

    def test_model_delete(self):
        """
        test delete protectionlayer deletes links
        """
        pl_c = self.pl_model.objects.count()
        me_c = self.me_model.objects.count()
        plme_c = self.plme_model.objects.count()
        ap_c = self.ap_model.objects.count()

        defaultuser = User.objects.get(pk=1)
        protectionlayer = self.pl_model(name='PL')
        protectionlayer.save(defaultuser, parent=None, treeelementkwargs={'scenario': self.scenario})
        plme = self.plme_model(protectionlayer=protectionlayer, measure=self.measure)
        plme.save(defaultuser)

        self.assertEquals(self.pl_model.objects.count(), pl_c+1)
        self.assertEquals(self.me_model.objects.count(), me_c)
        self.assertEquals(self.plme_model.objects.count(), plme_c+1)
        self.assertEquals(self.ap_model.objects.count(), ap_c)

        self.pl_model.objects.get(pk=protectionlayer.pk).delete(defaultuser)

        self.assertEquals(self.pl_model.objects.count(), pl_c)
        self.assertEquals(self.me_model.objects.count(), me_c)
        self.assertEquals(self.plme_model.objects.count(), plme_c)
        self.assertEquals(self.ap_model.objects.count(), ap_c)

    def test_protectionlayer_views(self):

        base_url = '/analyses/%s/protectionlayers/' % self.namespace
        parent_url = '/analyses/%s/scenarios/%s/causetree/' % (self.namespace, self.scenario.v_id)
        defaultuser = User.objects.get(pk=1)
        users = (0,1,2,3)


        class protectionlayers(utils_tests.TestDefinition):
            description = 'list of protection layers'
            url = base_url
            users = [1,2,]
            contains = [{'url': base_url + 'add/', 'users': []},]

        class protectionlayer(utils_tests.TestDefinition):
            description = 'display protectionlayer'
            url = base_url + '%s/' % self.protectionlayer.v_id
            users = [1,2,]
            contains = [
                {'url': base_url + '%s/edit/' % self.protectionlayer.v_id, 'users': [1,]},
                {'url': base_url + '%s/delete/' % self.protectionlayer.v_id, 'users': [1,]},
                {'url': base_url + 'measures/%s/delete/' % self.plme.v_id, 'users': [1,]},
            ]

        class protectionlayer_compare(utils_tests.TestDefinition):
            description = 'compare protectionlayer'
            url = base_url + '%s/compare/0/' % self.protectionlayer.v_id
            users = [1,2,]

        class protectionlayer_delete(utils_tests.delete_TestDefinition):
            description = 'delete protectionlayer'
            users = [1,]
            redirect_url = parent_url
            def pretest(self, testcase, allowed):
                newitem = self.test_class.pl_model(name='to_delete')
                newitem.save(defaultuser, treeelementkwargs={'scenario': self.test_class.scenario})
                self.pk = newitem.id
                self.url = base_url + '%s/delete/' % newitem.v_id
            def posttest(self, testcase, allowed):
                if not allowed:
                    self.test_class.pl_model.objects.get(id=self.pk).delete(defaultuser, override=True)

        tests = [
            protectionlayers(self),
            protectionlayer(self),
            protectionlayer_compare(self),
            protectionlayer_delete(self),
        ]

        utils_tests.run_tests(self, users, tests)

    def test_protectionlayer_overrideable_views(self):
        """
        Seperate so the deviations app can override these
        """
        base_url = '/analyses/%s/protectionlayers/' % self.namespace
        parent_url = '/analyses/%s/scenarios/%s/causetree/' % (self.namespace, self.scenario.v_id)
        defaultuser = User.objects.get(pk=1)
        users = (0,1,2,3)

        class protectionlayer_add(utils_tests.add_TestDefinition):
            description = 'add protectionlayer'
            url = '/analyses/%s/scenarios/%s/protectionlayers/add/' % (self.namespace, self.scenario.v_id)
            users = [1,]
            data = {
                "name": "bogus",
                "description": "",
                "management-referer": parent_url
            }
            redirect_url = parent_url
            def posttest(self, testcase, allowed):
                if allowed:
                    self.test_class.pl_model.objects.order_by('-id')[0].delete(testcase.user, override=True)

        class protectionlayer_edit(utils_tests.edit_TestDefinition):
            description = 'edit protectionlayer'
            url = base_url + '%s/edit/' % self.protectionlayer.v_id
            users = [1,]
            data = {
                "name": "Interlock",
                "description": "",
                "management-referer": base_url + '%s/' % self.protectionlayer.v_id
            }
            redirect_url = base_url + '%s/' % self.protectionlayer.v_id

        tests = [
            protectionlayer_add(self),
            protectionlayer_edit(self),
        ]

        utils_tests.run_tests(self, users, tests)

    def test_protectionlayer_measure_views(self):
        defaultuser = User.objects.get(pk=1)
        users = (0,1,2,3)

        class plme_delete(utils_tests.delete_TestDefinition):
            description = 'delete protectionlayer_measure link'
            users = [1,]
            redirect_url = '/analyses/%s/protectionlayers/%s/' % (self.namespace, self.protectionlayer.v_id)

            def pretest(self, testcase, allowed):
                newplme = self.test_class.plme_model(protectionlayer=self.test_class.protectionlayer, measure=self.test_class.measure2)
                newplme.save(defaultuser)
                self.pk = newplme.id
                self.url = '/analyses/%s/protectionlayers/measures/%s/delete/' % (self.test_class.namespace, newplme.v_id)
            def posttest(self, testcase, allowed):
                if not allowed:
                    self.test_class.plme_model.objects.get(id=self.pk).delete(defaultuser, override=True)

        tests = [
            plme_delete(self),
        ]

        utils_tests.run_tests(self, users, tests)



