"""
Hazards app

App for hazard analysis

"""
from django.conf import settings
from django.db import models
from django.db.models import signals as django_signals
from utils.strings import ugettext_tolanguage as _l
from django.utils.translation import ugettext_lazy as _
from utils.versioning import models as versioning_models
from utils import reports as utils_reports

class SubstanceManager(versioning_models.VersionedModelManager):
    def get_by_natural_key(self, name):
        return self.get(name=name, version__isnull=True)

class Substance(versioning_models.VersionedModel):
    name = models.CharField(verbose_name=_('Name'), max_length=100)
    tag = models.CharField(max_length=100, verbose_name = _('Tag'), blank=True, null=True)
    description = models.TextField(verbose_name = _('Description'), blank=True, null=True)
    hazards = models.TextField(verbose_name=_('Hazards'), blank=True, null=True)
    universal = models.BooleanField(verbose_name=_('Universal'), default=False)

    objects = SubstanceManager()

    class Meta:
        verbose_name = _('Substance')
        verbose_name_plural = _('Substances')
        permissions = (
            ("display_hazards", "May display hazards information"),
            ("edit_hazards", "May edit hazards information"),
        )

    def __unicode__(self):
        return self.name

    def natural_key(self):
        return (self.name,)

    class ReactionsDescriptor(object):
        def __get__(self, instance, instance_type=None):
            if instance is None:
                return self

            cache_name = 'reaction_set_cache'
            try:
                return getattr(instance, cache_name)
            except AttributeError:
                version = getattr(instance, '_version', instance.version) or versioning_models.WorkVersion
                res = Reaction.objects.filter(models.Q(substance1=instance)|models.Q(substance2=instance)).get_versionedobjects(version=version)
                setattr(instance, cache_name, res)
                return res
    reaction_set = ReactionsDescriptor()


    # urls

    @models.permalink
    def get_absolute_url(self, tab='substance'):
        return ('hazards_substance', (self.v_id, tab))

    @models.permalink
    def get_edit_url(self, tab='substance'):
        return ('hazards_substance_edit', (self.v_id,))

    @models.permalink
    def get_delete_url(self):
        return ('hazards_substance_delete', (self.v_id,))

    @models.permalink
    def get_report_url(self):
        return ('hazards_substance_report', (self.v_id,))

    @models.permalink
    def get_compare_url(self, version):
        return ('hazards_substance_compare', (self.v_id, version.id))


    # permissions

    @classmethod
    def may_display(cls, user):
        return user.has_perm('hazards.display_hazards')

    @classmethod
    def may_add(cls, user):
        return user.has_perm('hazards.edit_hazards')

    def may_change(self, user):
        if user.has_perm('hazards.edit_hazards'):
            return not self.version
        return False

    def may_delete(self, user):
        """
        Only allow delete if substance not referenced.
        """
        if user.has_perm('hazards.edit_hazards'):
            if not self.version:
                return self.reaction_set1.count() == 0 and self.reaction_set2.count() == 0 and self.section_substance_set.count() == 0
        return False

    @classmethod
    def get_listreport_formfields(cls, user):
        from django import forms
        res = [{'name': 'substances',
            'field': forms.BooleanField(required=False, label=_('Substance sheets'),),
            'children': cls.get_report_formfields(user)}]
        return res

    @classmethod
    def get_report_formfields(cls, user):
        return []

    def get_report_body(self, user, language=u'en', choices={}):
        from hazards import forms as app_forms
        from hazards import tables as app_tables
        form = app_forms.SubstanceSheet(instance=self, omit_empty=True)
        context = {'object': self, 'form': form, 'title': u'%s: %s' % (_l('Substance', language), self)}
        tables = []
        queryset = self.reaction_set.all().select_related_versioned()
        if queryset:
            table = app_tables.ReactionsTable(queryset)
            table.title = _l('Reactions', language)
            tables.append(table)
        context['tables'] = tables
        html = utils_reports.make_pdf_body(context, language=language)
        return [html,]

    @classmethod
    def get_report_css(cls, language=u'en', choices={}):
        return []

    def get_report(self, user, language=u'en', choices={}):
        body_strings = self.get_report_body(user, language, choices)
        css_files = self.__class__.get_report_css(language, choices)
        return utils_reports.make_pdf(body_strings, css_files, language)

class ReactionManager(versioning_models.VersionedModelManager):
    def get_by_natural_key(self, name):
        return self.get(name=name, version__isnull=True)

class Reaction(versioning_models.VersionedModel):
    name = models.CharField(verbose_name=_('Name'), max_length=100)
    description = models.TextField(verbose_name=_('Description'), blank=True, null=True)
    substance1 = versioning_models.VersionedForeignKey(Substance, verbose_name=_('Substance 1'), related_name='reaction_set1', blank=True, null=True)
    substance2 = versioning_models.VersionedForeignKey(Substance, verbose_name=_('Substance 2'), related_name='reaction_set2', blank=True, null=True)
    conditions = models.TextField(verbose_name=_('Reaction conditions'), blank=True, null=True)

    objects = ReactionManager()

    class Meta:
        verbose_name = _('Reaction')
        verbose_name_plural = _('Reactions')

    def __unicode__(self):
        return self.name

    def natural_key(self):
        return (self.name,)

    # urls

    @models.permalink
    def get_absolute_url(self, tab='reaction'):
        return ('hazards_reaction', (self.v_id, tab))

    @models.permalink
    def get_edit_url(self, tab='substance'):
        return ('hazards_reaction_edit', (self.v_id,))

    @models.permalink
    def get_delete_url(self):
        return ('hazards_reaction_delete', (self.v_id,))

    @models.permalink
    def get_report_url(self):
        return ('hazards_reaction_report', (self.v_id,))

    @models.permalink
    def get_compare_url(self, version):
        return ('hazards_reaction_compare', (self.v_id, version.id))


    # permissions

    @classmethod
    def may_display(cls, user):
        return user.has_perm('hazards.display_hazards')

    @classmethod
    def may_add(cls, user):
        return user.has_perm('hazards.edit_hazards')

    def may_change(self, user):
        if user.has_perm('hazards.edit_hazards'):
            return not self.version
        return False

    def may_delete(self, user):
        if user.has_perm('hazards.edit_hazards'):
            if not self.version:
                return self.section_reaction_set.count() == 0
        return False


    @classmethod
    def get_listreport_formfields(cls, user):
        from django import forms
        res = [{'name': 'reactions',
            'field': forms.BooleanField(required=False, label=_('Reaction sheets'),),
            'children': cls.get_report_formfields(user)}]
        return res

    @classmethod
    def get_report_formfields(cls, user):
        return []

    def get_report_body(self, user, language=u'en', choices={}):
        from hazards import forms as app_forms
        form = app_forms.ReactionSheet(instance=self, omit_empty=True)
        context = {'object': self, 'form': form, 'title': u'%s: %s' % (_l('Reaction', language), self)}
        html = utils_reports.make_pdf_body(context, language=language)
        return [html,]

    @classmethod
    def get_report_css(cls, language=u'en', choices={}):
        return []

    def get_report(self, user, language=u'en', choices={}):
        body_strings = self.get_report_body(user, language, choices)
        css_files = self.__class__.get_report_css(language, choices)
        return utils_reports.make_pdf(body_strings, css_files, language)


def show_matrix():
    return Substance.objects.filter(version__isnull=True).count() <= settings.HAZARDS_MATRIX_MAX_SUBSTANCES

def may_publish(user):
    return user.has_perm('hazards.edit_hazards') and user.has_perm('versioning.edit_versions')

def publish_hazards(user, instance):
    """
    Create a published version of the hazards app
    """

    objects = []

    filterkwarg={'published_dirty': True, 'version__isnull': True}

    # substances
    objects.append(list(Substance.objects.filter(**filterkwarg)))
    # reactions
    objects.append(list(Reaction.objects.filter(**filterkwarg)))

    instance.make_version_for_objects(objects)


def make_hazards_version(sender, instance, objects, **kwargs):
    if not objects:
        # only version everything when objects are not specifically included
        if instance.published:
            filterkwarg={'published_dirty': True, 'version__isnull': True}
        else:
            filterkwarg={'dirty': True, 'version__isnull': True}
        for item in Substance.objects.filter(**filterkwarg):
            item.make_version(instance)
        for item in Reaction.objects.filter(**filterkwarg):
            item.make_version(instance)

versioning_models.post_version.connect(make_hazards_version, sender=versioning_models.Version)

def restore_hazards_version(sender, instance, objects, **kwargs):
    if not objects:
        for item in Substance.objects.get_newerobjects(instance):
            item.restore_version(instance)
        for item in Reaction.objects.get_newerobjects(instance):
            item.restore_version(instance)

versioning_models.restore_version.connect(restore_hazards_version, sender=versioning_models.Version)


from utils.search.models import searches, Search

class Hazards_SubstanceSearch(Search):
    permission = 'hazards.display_hazards'
    fields = ['name', 'description', 'hazards']
    model = Substance

searches.register(Hazards_SubstanceSearch)

class Hazards_ReactionSearch(Search):
    permission = 'hazards.display_hazards'
    fields = ['name', 'description', 'conditions']
    model = Reaction

searches.register(Hazards_ReactionSearch)

