# reaction views

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from utils import crud
from utils import forms as utils_forms
from utils.autocomplete import autocomplete
from utils.versioning import versionedcrud
from utils.versioning.session import get_current_version
from utils.versioning.models import WorkVersion

from hazards import models as app_models
from hazards import forms as app_forms
from hazards import tables as app_tables
from hazards.views.main import hazards_navigationlist

from installations import models as installations_models
from installations import tables as installations_tables

class MixIn(object):
    "Common items for object views"
    model = app_models.Reaction
    include_attachments = True
    include_links = True

    def add_to_extra_context(self, extra_context, object):
        extra_context['navigationlist'] =  hazards_navigationlist()

    tabs = [
        {'name': 'reaction',
            'label': _('Properties')},
        {'name': 'sections',
            'label': _('Sections')},
    ]

class display_reaction(MixIn, versionedcrud.DisplayObjectView):
    form_class = app_forms.ReactionSheet
    def add_to_extra_context(self, extra_context, object):
        versionedcrud.DisplayObjectView.add_to_extra_context(self,extra_context, object)
        MixIn().add_to_extra_context(extra_context, object)

class display_sections(MixIn, versionedcrud.DisplayObjectView):
    allow_change = False
    allow_delete = False
    form_class = None

    def add_to_extra_context(self, extra_context, object):
        versionedcrud.DisplayObjectView.add_to_extra_context(self,extra_context, object)
        MixIn().add_to_extra_context(extra_context, object)
        queryset = object.section_set.all()
        table = installations_tables.SectionsTable(queryset)
        extra_context['table'] = table

class display(MixIn, crud.TabContainer):
    tabviews = {
        'reaction': display_reaction(),
        'sections': display_sections(),
    }
    default = 'reaction'

class edit(MixIn, versionedcrud.EditObjectView):
    form_class = app_forms.ReactionForm
    tabs = []

    def get_form(self, object):
        return super(edit, self).get_form(object, version=WorkVersion)

class delete(MixIn, versionedcrud.DeleteObjectView):
    form_class = app_forms.ReactionSheet
    tabs = []

    def get_default_url(self, object):
        return reverse('hazards_reactions')

class report(MixIn, versionedcrud.ReportObjectView):
    pass

class add(MixIn, versionedcrud.AddObjectView):
    form_class = app_forms.ReactionForm
    tabs = []

    def get_form(self, object):
        return super(add, self).get_form(object, version=WorkVersion)

class compare(MixIn, versionedcrud.CompareObjectView):
    form_class = app_forms.ReactionSheet

    def add_to_extra_context(self, extra_context, object):
        versionedcrud.CompareObjectView.add_to_extra_context(self,extra_context, object)
        MixIn().add_to_extra_context(extra_context, object)

def name_lookup(request):
    (version, special) = get_current_version(request)
    queryset = app_models.Reaction.objects.get_versionedobjects(version=version).values('name').order_by('name')
    return autocomplete(request, queryset, 'name', 'name')

class ReactionFilter(crud.Filter):
    form = app_forms.ReactionFilterForm
    filters = [
                {'filteritem': 'name', 'parameter': 'name__icontains', 'type': 'normal'},
            ]

class ReactionListObject(versionedcrud.TableListObject):
    defaultsort = 'name'
    table_class = app_tables.ReactionsTable
    queryset = app_models.Reaction.objects.all()
    title = _('Reactions')

    def get_query_set(self):
        # add special select_related
        return self.queryset.get_versionedobjects(self.version).select_related_versioned()

    def add_html(self, html_strings, queryset, user, language, choices):
        if choices.get('reactions'):
            for item in queryset:
                html_strings.append('<pdf:nextpage>')
                html_strings.extend(item.get_report_body(user, language, choices))

    def add_css(self, css_files, queryset, user, language, choices):
        if choices.get('reactions'):
            if queryset:
                css_files.extend(queryset[0].get_report_css(language, choices))

class reactions(versionedcrud.TableListView):
    "List of reactions"

    table_class = app_tables.ReactionsTable
    csv_table_class = app_tables.ReactionsCSVTable
    defaultsort = 'name'
    queryset = app_models.Reaction.objects.all()
    paginate_by = 100
    title = _('Reactions')
    filter = ReactionFilter()
    redirect_on_unique = True
    allow_pdf = True
    allow_csv = True
    pdfselectform_class = utils_forms.ListReportForm
    pdfsendmail = True
    listobject = ReactionListObject()

    def get_pdfselectform(self):
        if self.request.method == 'POST':
            form = self.get_pdfselectform_class()(data=self.request.POST, user=self.request.user)
        else:
            form = self.get_pdfselectform_class()(user=self.request.user)
        return form

    def get_pdfselectform_class(self):
        return utils_forms.create_class_with_meta(self.pdfselectform_class, model=app_models.Reaction)

    def add_to_extra_context(self, extra_context):
        super(reactions, self).add_to_extra_context(extra_context)
        extra_context['navigationlist'] =  hazards_navigationlist('reactions', self.request.user)
        tablebuttons = []
        if app_models.may_publish(self.request.user):
            tablebuttons.append({
                'url': reverse('hazards_publish'),
                'label': _('Publish')}
            )
        if (self.version==WorkVersion) and app_models.Reaction.may_add(self.request.user):
            tablebuttons.append({
                    'url': reverse('hazards_reaction_add'),
                    'label': _('Add')}
            )
        extra_context['tablebuttons'] = tablebuttons


class matrix(versionedcrud.NoObjectView):
    template = 'hazards/reactionmatrix.html'

    def comp(self, s1, s2):
        if s1.universal and not s2.universal:
            return 1
        if s2.universal and not s1.universal:
            return -1
        if s1.name < s2.name:
            return -1
        if s1.name == s2.name:
            return 0
        return 1

    def add_to_extra_context(self, extra_context):
        versionedcrud.NoObjectView.add_to_extra_context(self,extra_context)
        extra_context['navigationlist'] =  hazards_navigationlist('matrix', self.request.user)
        extra_context['title'] = _('Reaction matrix')
        substances = app_models.Substance.objects.all().get_versionedobjects(self.version).order_by('-universal', '-name')
        reactions = app_models.Reaction.objects.all().get_versionedobjects(self.version).select_related_versioned()
        reaction_dict = {}
        for int in reactions:
            if int.substance1 and int.substance2:
                if self.comp(int.substance1, int.substance2) < 0:
                    reaction_dict.setdefault((int.substance1.id, int.substance2.id),[]).append(int)
                else:
                    reaction_dict.setdefault((int.substance2.id, int.substance1.id),[]).append(int)

        rows = []
        count = substances.count()
        i = 0
        for item in substances.order_by('universal', 'name'):
            x = []
            j = 0
            for sub in substances:
                r = reaction_dict.get((item.id, sub.id), [])
                a = u''
                if r:
                    a = u'used'
                elif j <= (count - i - 1):
                    a = u'available'
                x.append({'reactions': r, 'class': a})
                j += 1
            rows.append({'substance': item, 'cell': x})
            i += 1
        matrix = {'substances': substances, 'rows': rows}
        extra_context['matrix'] = matrix
