# main items for installations app

from analyses.main import analysesmodules

from django.conf import settings
from django.contrib import messages
from django.core.urlresolvers import reverse
from django import forms
from django.http import HttpResponse
from django.shortcuts import get_object_or_404
from django.utils import simplejson
from django.utils.translation import ugettext_lazy as _

from installations import models as app_models
from installations import forms as app_forms
from installations import tables as app_tables

from utils import crud, permissions
from utils import reports as utils_reports
from utils.trees import models as trees_models
from utils.trees import views as trees_views
from utils.versioning import versionedcrud
from utils.versioning.helpers import annotate_versioned
from utils.versioning.models import Version, WorkVersion
from utils.versioning.session import get_current_version

def installations_navigationlist(current=None, user=None):
    if current == 'overview':
        return [{'label': _('Site Breakdown Structure__')}]
    return [{'url': reverse('installations_overview'),
            'label': _('Site Breakdown Structure__')}]

def local_navigationlist(current=None, user=None):
    return [
        { 'url': reverse('installations_scopes'),
        'label': _('Scopes')}
    ]

def activationlist(current=None):
    items = analysesmodules.get_items_sorted()
    res = []
    for item in items:
        if item['appconfig'].namespace == current:
            res.append({'label': item['appconfig'].analysistype_short})
        else:
            url = reverse('%s_activate' % item['appconfig'].namespace)
            res.append({'url': url, 'label': item['appconfig'].analysistype_short})
    return res


def get_visible_objects(queryset):
    """
    takes a queryset of objects that are annotated with a selected attribute
    returns a list of the visible objects
    """
    # determine which nodes must be open because they are ancestor to a selected node
    queryset = list(queryset)
    queryset.reverse()
    last_level = 0
    open_level = 0
    must_open = False
    for item in queryset:
        item.must_open = False
        # moving to higher level: must be open too
        if must_open and (item.level < open_level):
            item.must_open = True
        # from here on only higher levels must be opened, not siblings
            open_level -= 1
        if item.selected:
            # only set open_level if there was not a child selected yet
            # or reset open_level if we found a selected item on a lower level
            if not must_open or (item.level > open_level):
                open_level = item.level
            must_open = True
        last_level = item.level
    queryset.reverse()
    # include all levels up to max_level
    # if an item is must_open: all its children must be included too
    res = []
    max_level = settings.INSTALLATIONS_OVERVIEW_DEFAULT_LEVEL
    open_level = 0
    must_open = False
    for item in queryset:
        # moving to higher level: turn off
        if item.level <= open_level:
            must_open = False
        # the item is marked: set flag and include the item
        if item.must_open:
            must_open = True
            open_level = item.level
            res.append(item)
        else:
            # child item: include if must_open
            if must_open and (item.level == open_level+1):
                item.max_level = not item.is_leaf_node()
                res.append(item)
            # if not: include if max_level not reached
            if not must_open:
                if (item.level <= max_level):
                    item.max_level = item.level == max_level
                    res.append(item)
    return res

class overview(versionedcrud.NoObjectView):
    template = 'installations/overview.html'

    def add_to_extra_context(self, extra_context):
        versionedcrud.NoObjectView.add_to_extra_context(self,extra_context)
        sections = None
        name = None
        analysistype = self.request.session.get('analysistype')
        initial = {}
        if analysistype:
            initial['analysistype'] = analysistype
        if self.request.method == 'POST':
            if self.request.POST.get('submit_clear'):
                filterform = app_forms.OverviewFilterForm(prefix='filter')
            else:
                filterform = app_forms.OverviewFilterForm(data=self.request.POST, prefix='filter')
            if filterform.is_valid():
                name = filterform.cleaned_data['name']
                analysistype = filterform.cleaned_data['analysistype']
        else:
            filterform = app_forms.OverviewFilterForm(prefix='filter', initial=initial)
        if analysistype:
            analysismodel = analysesmodules[analysistype].get_model('scenario')
            if analysismodel:
                scopes = analysismodel.objects.filter(scope__isnull=False).order_by('scope').get_versionedobjects(version=self.version).values_list('scope', flat=True).distinct()
                if scopes:
                    sections = app_models.Scope_Section.objects.filter(scope__in=scopes).get_versionedobjects(version=self.version).values_list('section', flat=True)

        extra_context['activationlist'] =  activationlist(analysistype)
        extra_context['navigationlist'] =  local_navigationlist('overview', self.request.user)

        extra_context['title'] = _('Site Breakdown Structure')
        queryset = app_models.SectionTreeElement.tree.all().get_versionedobjects(self.version)
        if name:
            # we need the section name when name is selected
            queryset = queryset.select_related_versioned()
        else:
            # add the section info if this was not yet done
            annotate_versioned(queryset, 'section')

        for item in queryset:
            item.selected = False
            if sections is not None:
                if item.section.v_id in sections:
                    if name:
                        if name in item.section.name:
                            item.selected = True
                    else:
                        item.selected = True
            else:
                if name:
                    if name.lower() in item.section.name.lower():
                        item.selected = True
        res = get_visible_objects(queryset)

        extra_context['objects'] = res
        buttons = []
        if app_models.may_publish(self.request.user):
            buttons.append(
                    {'url': reverse('installations_publish'),
                        'label': _('Publish')})
        if app_models.SectionTreeElement.may_change(self.request.user):
            buttons.append(
                    {'url': reverse('installations_overview_edit'),
                        'label': _('Change')})
        extra_context['buttons'] = buttons
        extra_context['filterform'] = filterform
        extra_context['helptext'] = _('Enter part of a name to search a section.\nSelect a safety function to search for sections for which scenarios of that safety function are available.')

        # history
        if self.request.user.has_perm('versioning.display_versions'):
            extra_context['history_url'] = reverse('installations_overview_history')
            versionids = list(app_models.SectionTreeElement.objects.filter(parent__isnull=True, version__isnull=False).values_list('version_id', flat=True).distinct())
            versionids.sort()
            versionids = versionids[-1:]
            if versionids:
                versions = Version.objects.filter(id__in=versionids).order_by('-id')
            else:
                versions = Version.objects.none()
            history = []
            for item in versions:
                history.append(
                    { 'version': item,
                    'url': reverse('installations_overview_compare', args=[item.id,])
                    })
            extra_context['history'] = history

            dirty = False
            roots = app_models.SectionTreeElement.objects.filter(parent__isnull=True).get_versionedobjects(self.version)
            for item in roots:
                if item.dirty:
                    dirty = True
            extra_context['dirty'] = dirty

class overview_edit(trees_views.edit_tree):
    treeelement_class = app_models.SectionTreeElement
    template = 'installations/overview_edit.html'
    helptext = _('Drag and drop the sections to rearrange the tree. Right-click to open the context menu.')

    def check_permission(self, object):
        if (self.version!=WorkVersion) or (not app_models.SectionTreeElement.may_change(self.request.user)):
            return permissions.redirect_to_not_allowed()

    def lookup_object(self,*args, **kwargs):
        # trick the view because we have no object
        (self.version, self.specialversion) = get_current_version(self.request)
        return None

    def get_title(self, object):
        # trick the view because we have no object
        return _('Edit the Site Breakdown Structure')

    def set_message(self, object):
        # trick the view because we have no object
        message = self.message
        if not message:
            message = _('Site Breakdown Structure modified')
        messages.info(self.request, message)

    def create_element(self, parent, type, name, treeelementkwargs={}):
        if type == 'section':
            instance = app_models.Section(name=name)
            return instance.save(self.request.user, parent=parent)

    def rename_element(self, element, name):
        element.section.name = name
        element.section.save(self.request.user)

    def get_treeobjects(self, object):
        return app_models.SectionTreeElement.get_authorized_tree(self.request.user)

class overview_compare(versionedcrud.NoObjectView):
    template = 'installations/overview_compare.html'

    def __call__(self, request,version_id=None, *args, **kwargs):
        self.version_id = version_id
        return super(overview_compare, self).__call__(request, *args, **kwargs)

    def setup_from_args(self, *args, **kwargs):
        (self.version, self.specialversion) = get_current_version(self.request)
        # make 0 into WorkVersion
        if self.version_id and int(self.version_id):
            self.version2 = get_object_or_404(Version, id = self.version_id)
        else:
            self.version2 = WorkVersion

    def add_to_extra_context(self, extra_context):
        versionedcrud.NoObjectView.add_to_extra_context(self,extra_context)
        extra_context['navigationlist'] =  installations_navigationlist()
        extra_context['title'] = _('Site Breakdown Structure')
        extra_context['objects'] = app_models.SectionTreeElement.tree.all().get_versionedobjects(self.version).select_related_versioned()
        extra_context['objects2'] = app_models.SectionTreeElement.tree.all().get_versionedobjects(self.version2).select_related_versioned()
        extra_context['version2'] = self.version2
        buttons = [{
                    'url': reverse('installations_overview'),
                    'label': _('Back')}
            ]
        extra_context['buttons'] = buttons

class publish(versionedcrud.PublishObjectView):
    def lookup_object(self, *args, **kwargs):
        (self.version, self.specialversion) = get_current_version(self.request)
        return None

    def check_permission(self, object):
        if not app_models.may_publish(self.request.user):
            return permissions.redirect_to_not_allowed()

    def get_name(self, object):
        return _('Site Breakdown Structure')

    def save_form(self, form):
        name = form.cleaned_data['name']
        instance = Version(name=name, published=True)
        instance.save(self.request.user, propagate=False)
        app_models.publish_installations(self.request.user, instance)
        return self.object

    def add_to_extra_context(self, extra_context, object):
        extra_context['helptext'] = _('Publishing the installations information will NOT include the scenario information inside the sections!')

class overview_history(versionedcrud.NoObjectView):
    template = 'installations/overview_history.html'
    permission = 'versioning.display_versions'
    def add_to_extra_context(self, extra_context):
        versionedcrud.NoObjectView.add_to_extra_context(self,extra_context)
        extra_context['navigationlist'] =  installations_navigationlist()
        extra_context['title'] = _('Site Breakdown Structure history')
        versionids = list(app_models.SectionTreeElement.objects.filter(parent__isnull=True, version__isnull=False).values_list('version_id', flat=True).distinct())
        if versionids:
            versions = Version.objects.filter(id__in=versionids).order_by('-id')
        else:
            versions = Version.objects.none()
        history = []
        for item in versions:
            history.append(
                { 'version': item,
                'url': reverse('installations_overview_compare', args=[item.id,])
                })
        extra_context['history'] = history

        dirty = False
        roots = app_models.SectionTreeElement.objects.filter(parent__isnull=True).get_versionedobjects(self.version)
        for item in roots:
            if item.dirty:
                dirty = True
        extra_context['dirty'] = dirty

        extra_context['buttons'] = [
            {'url': reverse('installations_overview'),
                'label': _('Back')},
        ]

class overview_report(versionedcrud.ReportNoObjectView):
    form_class = app_forms.OverviewReportForm
    pdfsendmail = True

    def get_title(self):
        return _('Site Breakdown Structure report')

    def get_form(self):
        if self.request.method == 'POST':
            form = self.form_class(data=self.request.POST, user=self.request.user)
        else:
            form = self.form_class(user=self.request.user)
        return form

    def setup_from_args(self, *args, **kwargs):
        super(overview_report, self).setup_from_args(*args, **kwargs)

    def set_task(self):
        from installations.tasks import mail_overview_report
        if not self.version or (self.version == WorkVersion):
            version_id = None
        else:
            version_id = self.version.pk
        mail_overview_report.delay(self.request.user, self.language, self.choices, version_id)

    def get_pdf(self):
        return get_overview_report(self.request.user, self.language, self.choices, self.version)

def get_overview_report_formfields(user):
    res = [{'name': 'sections',
        'field': forms.BooleanField(required=False, label=_('Section sheets'),),
        'children': analysesmodules.get_report_formfields(user)}]
    return res

def get_overview_report(user, language=u'en', choices={}, version=WorkVersion):
    sections = app_models.SectionTreeElement.tree.all().get_versionedobjects(version)
    context = {
        'title': _('Site Breakdown Structure'),
        'objects': sections
    }
    html = [utils_reports.make_pdf_body(context, language=language, template='installations/overview_report.html')]
    if choices.get('sections'):
        for s in sections:
            html.append('<pdf:nextpage>')
            html.extend(s.section.get_report_body(user, language, choices, subsection=True))
    css_files = []
    for mod in analysesmodules.get_modules():
        css_files.extend(mod.get_model('scenario').get_report_css(language, choices))
    return utils_reports.make_pdf(html, css_files, language)

def tree_json(request):
    (version, special) = get_current_version(request)
    objects = []
    q = request.GET.get("id")
    if q:
        object = app_models.Section.objects.get_versionedobject(version=version, v_id=int(q))
        if object:
            el = object.sectiontreeelement_set.all()[0]
            for item in app_models.SectionTreeElement.tree.filter(parent=el).get_versionedobjects(version=version).select_related_versioned():
                object = {
                    "data": {
                        "title" : u'%s' % item.section,
                        "attr": { "href": item.section.get_absolute_url()}
                    },
                    "attr": {
                        "id" : item.section.v_id
                    }
                }
                if not item.is_leaf_node():
                    object['state'] = u'closed'
                objects.append(object)
    json = simplejson.dumps(objects)
    return HttpResponse(json, mimetype='application/json')
