"""
Tasks app

! Depends on the versioning app

Objects for now need to be VersionedModel subclasses

"""
import datetime
from django.db import models
from django.db.models import signals as django_signals
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext_lazy as _

from utils import reports as utils_reports
from utils.versioning import models as versioning_models

class TaskManager(versioning_models.VersionedModelManager):
    def tasks_for_object(self, obj):
        object_type = ContentType.objects.get_for_model(obj)
        version = getattr(obj, '_version', obj.version) or versioning_models.WorkVersion
        return self.filter(content_type__pk=object_type.id,
                           object_id=obj.v_id).get_versionedobjects(version=version)

    def open_tasks_for_object(self, obj):
        object_type = ContentType.objects.get_for_model(obj)
        version = getattr(obj, '_version', obj.version) or versioning_models.WorkVersion
        return self.filter(content_type__pk=object_type.id,
                           object_id=obj.v_id,
                           completiondate__isnull=True).get_versionedobjects(version=version)

    def late_tasks_for_object(self, obj):
        object_type = ContentType.objects.get_for_model(obj)
        version = getattr(obj, '_version', obj.version) or versioning_models.WorkVersion
        return self.filter(content_type__pk=object_type.id,
                           object_id=obj.v_id,
                           completiondate__isnull=True,
                           duedate__lt=datetime.datetime.now()).get_versionedobjects(version=version)

class Task(versioning_models.VersionedModel):
    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = versioning_models.VersionedGenericForeignKey('content_type', 'object_id')
    task = models.CharField(max_length=100, verbose_name = _('Task'))
    description = models.TextField(verbose_name=_('Description'), null=True, blank=True)
    result = models.TextField(verbose_name=_('Result'), null=True, blank=True)
    duedate = models.DateField(verbose_name = _('Due date'), null=True, blank=True)
    completiondate = models.DateField(verbose_name = _('Completion date'), null=True, blank=True)
    user = models.ForeignKey(User, verbose_name = _('Responsable'), null=True, blank=True)

    objects = TaskManager()

    class Meta:
        verbose_name = _('Task')
        verbose_name_plural = _('Tasks')
        ordering = ['duedate',]
        permissions = (
            ("display_tasks", "May display tasks"),
            ("edit_tasks", "May edit tasks"),
            ("get_tasks", "May get a task"),
        )

    def __unicode__(self):
        return '%s' % self.task


    # urls

    @models.permalink
    def get_absolute_url(self):
        return ('tasks_task_display', (self.v_id,))

    @models.permalink
    def get_edit_url(self):
        return ('tasks_task_edit', (self.v_id,))

    @models.permalink
    def get_delete_url(self):
        return ('tasks_task_delete', (self.v_id,))

    @models.permalink
    def get_report_url(self):
        return ('tasks_task_report', (self.v_id,))

    # permissions

    @classmethod
    def may_display(cls, user):
        return user.has_perm('tasks.display_tasks')

    @classmethod
    def may_add(cls, user, parent=None):
        return user.has_perm('tasks.edit_tasks')

    def may_change(self, user):
        if self.version:
            return False
        return user.has_perm('tasks.edit_tasks')

    def may_delete(self, user):
        return self.may_change(user)

    @classmethod
    def get_listreport_formfields(cls, user):
        from django import forms
        res = [{'name': 'tasks',
            'field': forms.BooleanField(required=False, label=_('Task sheets'),),
            'children': cls.get_report_formfields(user)}]
        return res

    @classmethod
    def get_report_formfields(cls, user):
        return []

    def get_report_body(self, user, language=u'en', choices={}):
        from tasks import forms as app_forms
        form = app_forms.TaskSheet(instance=self, omit_empty=True, report=True)
        context = {'object': self, 'form': form, 'title': u'%s: %s' % (_('Task'), self)}
        html = utils_reports.make_pdf_body(context, language=language)
        return [html,]

    @classmethod
    def get_report_css(cls, language=u'en', choices={}):
        return []

    def get_report(self, user, language=u'en', choices={}):
        body_strings = self.get_report_body(user, language, choices)
        css_files = self.__class__.get_report_css(language, choices)
        return utils_reports.make_pdf(body_strings, css_files, language)


def make_tasks_version(sender, instance, objects, **kwargs):
    if instance.published:
        filterkwarg={'published_dirty': True, 'version__isnull': True}
    else:
        filterkwarg={'dirty': True, 'version__isnull': True}
    if objects:
        for typelist in objects:
            if typelist:
                object_type = ContentType.objects.get_for_model(typelist[0])
                for item in Task.objects.filter(**filterkwarg).filter(content_type__pk=object_type.id,
                           object_id__in=[item.v_id for item in typelist]):
                    item.make_version(instance)
    else:
        for item in Task.objects.filter(**filterkwarg):
            item.make_version(instance)

versioning_models.post_version.connect(make_tasks_version, sender=versioning_models.Version)

def restore_tasks_version(sender, instance, objects, **kwargs):
    if objects:
        for typelist in objects:
            if typelist:
                object_type = ContentType.objects.get_for_model(typelist[0])
                for item in Task.objects.filter(content_type__pk=object_type.id,
                           object_id__in=[item.v_id for item in typelist]):
                    item.restore_version(instance)
    else:
        for item in Task.objects.get_newerobjects(instance):
            item.restore_version(instance)

versioning_models.restore_version.connect(restore_tasks_version, sender=versioning_models.Version)

from utils.search.models import searches, Search

class Tasks_TaskSearch(Search):
    permission = 'tasks.display_tasks'
    fields = ['task', 'description']
    model = Task

searches.register(Tasks_TaskSearch)
