"""
Tests for attachments app
"""

import datetime

from django import test
from django.conf import settings
from django.contrib.auth.models import User, Permission
from django.contrib.contenttypes.models import ContentType
from django.core.files.uploadedfile import SimpleUploadedFile

import time

import utils.attachments.attachments_test.models as test_models
from utils.attachments.models import Attachment
import utils.attachments.views as test_views
from utils import tests as utils_tests
from utils.versioning.models import Version, WorkVersion
from utils.versioning.versioning_test import models as vt_models

class AttachmentsTestCase(test.TestCase):
    fixtures = ['dummyusers',]

    def setUp(self):
        # give the users permissions

        # authorisations for users:
        # 1 = authorised
        # 2 = unauthorised to edit
        # 3 = unauthorised to even display

        u = User.objects.get(pk=1)

        permissions = [
            ('display_versions','versioning','version'),
        ]
        for item in permissions:
            u.user_permissions.add(Permission.objects.get_by_natural_key(*item))

        u = User.objects.get(pk=2)
        permissions = [
            ('display_versions','versioning','version'),
        ]
        for item in permissions:
            u.user_permissions.add(Permission.objects.get_by_natural_key(*item))

        s = test_models.AttachmentStreet(name='First')
        s.save(User.objects.get(pk=1))

class test_attachments(AttachmentsTestCase):

    def test_views(self):
        csid = test_models.AttachmentStreet.objects.all()[0].id
        csvid = test_models.AttachmentStreet.objects.all()[0].v_id

        self.assertEquals(test_views.add_url_for_obj(test_models.AttachmentStreet.objects.get(pk=csid)), "/attachments/add-for/attachments_test/attachmentstreet/%s/" % csvid)
        self.assertEquals(test_views.display_url_for_obj(test_models.AttachmentStreet.objects.get(pk=csid)), "/attachments/display-for/attachments_test/attachmentstreet/%s/" % csvid)

    def test_attachment_views(self):
        base_url = '/attachments/'
        defaultuser = User.objects.get(pk=1)
        users = (0,1,2)

        csid = test_models.AttachmentStreet.objects.all()[0].id
        csvid = test_models.AttachmentStreet.objects.all()[0].v_id

        a = Attachment(content_object=test_models.AttachmentStreet.objects.all()[0], attachment_file = SimpleUploadedFile('assignment.txt', 'content') )
        a.save(defaultuser)

        class attachment_add(utils_tests.add_TestDefinition):
            description = 'add an attachment'
            url = base_url + 'add-for/attachments_test/attachmentstreet/%s/' % csvid
            users = [1,]
            def pretest(self, testcase, allowed):
                # we need a new file object each time around
                self.data = {
                    "management-referer": base_url + 'display-for/attachments_test/attachmentstreet/%s/' % csvid,
                    "attachment_file": SimpleUploadedFile('assignment.txt', 'content'),
                }
            redirect_url = base_url + 'display-for/attachments_test/attachmentstreet/%s/' % csvid
            def posttest(self, testcase, allowed):
                if allowed:
                    Attachment.objects.order_by('-id').get_versionedobjects(version=WorkVersion)[0].delete(testcase.user, override=True)

        class attachment_delete(utils_tests.delete_TestDefinition):
            description = 'delete an attachment'
            users = [1,]
            redirect_url = base_url + 'display-for/attachments_test/attachmentstreet/%s/' % csvid
            def pretest(self, testcase, allowed):
                # create a new item that we can delete
                newitem = Attachment(attachment_file=SimpleUploadedFile('assignment.txt', 'content'), content_object=test_models.AttachmentStreet.objects.get(pk=csid))
                newitem.save(defaultuser)
                self.pk = newitem.id
                self.url = base_url + '%s/delete/' % newitem.v_id
            def posttest(self, testcase, allowed):
                # clean up if the item was not deleted
                if not allowed:
                    Attachment.objects.get_versionedobject(id=self.pk, version=WorkVersion).delete(defaultuser, override=True)

        class attachments_display(utils_tests.TestDefinition):
            description = 'display attachments for street 1'
            url = base_url + 'display-for/attachments_test/attachmentstreet/%s/' % csvid
            users = [1,2,]
            contains = [
                {'url': base_url + 'add-for/attachments_test/attachmentstreet/%s/' % csvid, 'users': [1,]},
                {'url': base_url + '%s/delete/' % a.v_id, 'users': [1,]},
            ]


        tests = [
            attachment_add(),
            attachment_delete(),
            attachments_display(),
        ]

        utils_tests.run_tests(self, users, tests)


    def test_templatetags(self):
        from django.template import Context, Template
        csid = test_models.AttachmentStreet.objects.all()[0].id

        d = {'MEDIA_URL': 'x', 'object': test_models.AttachmentStreet.objects.get(pk=csid)}

        t = Template("""
            {% load attachments %}
            {% attachments_link object %}
        """)
        response = t.render(Context(d))

class AttachmentsPublishTestCase(AttachmentsTestCase):
    def test_publish(self):
        """
        Test that when publishing a single object, the attachments for that object are also published
        """
        user = User.objects.get(pk=1)
        csid = test_models.AttachmentStreet.objects.all()[0].id
        csvid = test_models.AttachmentStreet.objects.all()[0].v_id

        self.assertEquals(vt_models.Building.objects.count(), 0)
        self.assertEquals(vt_models.Building.objects.next_v_id(), 1)

        b1 = vt_models.Building(name='b1')
        b1.save(user)

        b2 = vt_models.Building(name='b2')
        b2.save(user)

        self.assertEquals(vt_models.Building.objects.count(), 2)
        self.assertEquals(vt_models.Building.objects.dirty().count(), 2)
        self.assertEquals(vt_models.Building.objects.filter(published_dirty=True, version__isnull=True).count(), 2)
        self.assertEquals(vt_models.Building.objects.filter(published=True).count(), 0)

        v1 = Version(name='v1', published=True)
        v1.save(user)

        self.assertEquals(vt_models.Building.objects.count(), 4)
        self.assertEquals(vt_models.Building.objects.dirty().count(), 0)
        self.assertEquals(vt_models.Building.objects.filter(published_dirty=True, version__isnull=True).count(), 0)
        self.assertEquals(vt_models.Building.objects.filter(published=True).count(), 2)

        # add attachment to b1
        b1 = vt_models.Building.objects.get_versionedobject(v_id = 1, version=WorkVersion)
        a = Attachment(content_object=b1, attachment_file = SimpleUploadedFile('assignment.txt', 'content') )
        a.save(user)

        # add attachment to b2
        b2 = vt_models.Building.objects.get_versionedobject(v_id = 2, version=WorkVersion)
        a = Attachment(content_object=b2, attachment_file = SimpleUploadedFile('assignment.txt', 'content') )
        a.save(user)

        self.assertEquals(Attachment.objects.count(), 2)
        self.assertEquals(Attachment.objects.dirty().count(), 2)
        self.assertEquals(Attachment.objects.filter(published_dirty=True, version__isnull=True).count(), 2)
        self.assertEquals(Attachment.objects.filter(published=True).count(), 0)

        # create a version of b1
        time.sleep(0.01)
        v2 = Version(name='v2', published=True)
        v2.save(user, propagate=False)
        b2.publish(user, v2)

        self.assertEquals(Attachment.objects.count(), 3)
        self.assertEquals(Attachment.objects.dirty().count(), 1)
        self.assertEquals(Attachment.objects.filter(published_dirty=True, version__isnull=True).count(), 1)
        self.assertEquals(Attachment.objects.filter(published=True).count(), 1)

        # clean up (removes files)
        for att in Attachment.objects.all():
            att.version = None
            att.delete(user, override=True)